/**
 *  @file       punc_base.h
 *  @brief      The base (or foundation) header for PUNC.
 *  @version    $Id: punc_base.h,v 1.32 2010/08/12 05:52:19 fetk Exp $
 *  @author     Michael Holst
 *  
 *  @attention
 *  @verbatim
 *
 * PUNC = < Portable Understructure for Numerical Computing >
 * Copyright (C) 1994-- Michael Holst
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 *  @endverbatim
 */


#ifndef _PUNC_BASE_H_
#define _PUNC_BASE_H_

#include <maloc/maloc.h>

/** @brief Inlining via macros for speed */
#if 1
#   define VINLINE_PUNC
#endif

#endif /* _PUNC_BASE_H_ */

